/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#if ENABLE(INDEXED_DATABASE) && ENABLE(DATABASE_PROCESS)

#include "DatabaseProcessIDBConnection.h"

#include "ArgumentCoders.h"
#include "DataReference.h"
#include "DatabaseProcessIDBConnectionMessages.h"
#include "HandleMessage.h"
#include "MessageDecoder.h"
#include "SecurityOriginData.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/IDBIndexMetadata.h>
#include <WebCore/IDBKeyData.h>
#include <WebCore/IDBKeyRangeData.h>
#include <WebCore/IDBObjectStoreMetadata.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace DatabaseProcessIDBConnection {

ResetTransactionSync::DelayedReply::DelayedReply(PassRefPtr<IPC::Connection> connection, std::unique_ptr<IPC::MessageEncoder> encoder)
    : m_connection(connection)
    , m_encoder(WTF::move(encoder))
{
}

ResetTransactionSync::DelayedReply::~DelayedReply()
{
    ASSERT(!m_connection);
}

bool ResetTransactionSync::DelayedReply::send(bool success)
{
    ASSERT(m_encoder);
    *m_encoder << success;
    bool _result = m_connection->sendSyncReply(WTF::move(m_encoder));
    m_connection = nullptr;
    return _result;
}

RollbackTransactionSync::DelayedReply::DelayedReply(PassRefPtr<IPC::Connection> connection, std::unique_ptr<IPC::MessageEncoder> encoder)
    : m_connection(connection)
    , m_encoder(WTF::move(encoder))
{
}

RollbackTransactionSync::DelayedReply::~DelayedReply()
{
    ASSERT(!m_connection);
}

bool RollbackTransactionSync::DelayedReply::send(bool success)
{
    ASSERT(m_encoder);
    *m_encoder << success;
    bool _result = m_connection->sendSyncReply(WTF::move(m_encoder));
    m_connection = nullptr;
    return _result;
}

} // namespace DatabaseProcessIDBConnection

} // namespace Messages

namespace WebKit {

void DatabaseProcessIDBConnection::didReceiveDatabaseProcessIDBConnectionMessage(IPC::Connection& connection, IPC::MessageDecoder& decoder)
{
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::EstablishConnection::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::EstablishConnection>(decoder, this, &DatabaseProcessIDBConnection::establishConnection);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::GetOrEstablishIDBDatabaseMetadata::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::GetOrEstablishIDBDatabaseMetadata>(decoder, this, &DatabaseProcessIDBConnection::getOrEstablishIDBDatabaseMetadata);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::DeleteDatabase::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::DeleteDatabase>(decoder, this, &DatabaseProcessIDBConnection::deleteDatabase);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::OpenTransaction::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::OpenTransaction>(decoder, this, &DatabaseProcessIDBConnection::openTransaction);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::BeginTransaction::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::BeginTransaction>(decoder, this, &DatabaseProcessIDBConnection::beginTransaction);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::CommitTransaction::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::CommitTransaction>(decoder, this, &DatabaseProcessIDBConnection::commitTransaction);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::ResetTransaction::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::ResetTransaction>(decoder, this, &DatabaseProcessIDBConnection::resetTransaction);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::RollbackTransaction::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::RollbackTransaction>(decoder, this, &DatabaseProcessIDBConnection::rollbackTransaction);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::ChangeDatabaseVersion::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::ChangeDatabaseVersion>(decoder, this, &DatabaseProcessIDBConnection::changeDatabaseVersion);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::CreateObjectStore::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::CreateObjectStore>(decoder, this, &DatabaseProcessIDBConnection::createObjectStore);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::DeleteObjectStore::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::DeleteObjectStore>(decoder, this, &DatabaseProcessIDBConnection::deleteObjectStore);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::ClearObjectStore::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::ClearObjectStore>(decoder, this, &DatabaseProcessIDBConnection::clearObjectStore);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::CreateIndex::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::CreateIndex>(decoder, this, &DatabaseProcessIDBConnection::createIndex);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::DeleteIndex::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::DeleteIndex>(decoder, this, &DatabaseProcessIDBConnection::deleteIndex);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::PutRecord::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::PutRecord>(decoder, this, &DatabaseProcessIDBConnection::putRecord);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::GetRecord::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::GetRecord>(decoder, this, &DatabaseProcessIDBConnection::getRecord);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::OpenCursor::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::OpenCursor>(decoder, this, &DatabaseProcessIDBConnection::openCursor);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::CursorAdvance::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::CursorAdvance>(decoder, this, &DatabaseProcessIDBConnection::cursorAdvance);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::CursorIterate::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::CursorIterate>(decoder, this, &DatabaseProcessIDBConnection::cursorIterate);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::Count::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::Count>(decoder, this, &DatabaseProcessIDBConnection::count);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::DeleteRange::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::DeleteRange>(decoder, this, &DatabaseProcessIDBConnection::deleteRange);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::Close::name()) {
        IPC::handleMessage<Messages::DatabaseProcessIDBConnection::Close>(decoder, this, &DatabaseProcessIDBConnection::close);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

void DatabaseProcessIDBConnection::didReceiveSyncDatabaseProcessIDBConnectionMessage(IPC::Connection& connection, IPC::MessageDecoder& decoder, std::unique_ptr<IPC::MessageEncoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::ResetTransactionSync::name()) {
        IPC::handleMessageDelayed<Messages::DatabaseProcessIDBConnection::ResetTransactionSync>(connection, decoder, replyEncoder, this, &DatabaseProcessIDBConnection::resetTransactionSync);
        return;
    }
    if (decoder.messageName() == Messages::DatabaseProcessIDBConnection::RollbackTransactionSync::name()) {
        IPC::handleMessageDelayed<Messages::DatabaseProcessIDBConnection::RollbackTransactionSync>(connection, decoder, replyEncoder, this, &DatabaseProcessIDBConnection::rollbackTransactionSync);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

#endif // ENABLE(INDEXED_DATABASE) && ENABLE(DATABASE_PROCESS)
