/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DatabaseProcessIDBConnectionMessages_h
#define DatabaseProcessIDBConnectionMessages_h

#if ENABLE(INDEXED_DATABASE) && ENABLE(DATABASE_PROCESS)

#include "Arguments.h"
#include "MessageEncoder.h"
#include "StringReference.h"
#include <WebCore/IDBKeyData.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>

namespace IPC {
    class DataReference;
    class Connection;
}

namespace WTF {
    class String;
}

namespace WebCore {
    struct IDBKeyData;
    struct IDBObjectStoreMetadata;
    struct IDBIndexMetadata;
    struct IDBKeyRangeData;
}

namespace WebKit {
    struct SecurityOriginData;
}

namespace Messages {
namespace DatabaseProcessIDBConnection {

static inline IPC::StringReference messageReceiverName()
{
    return IPC::StringReference("DatabaseProcessIDBConnection");
}

class EstablishConnection {
public:
    typedef std::tuple<String, WebKit::SecurityOriginData, WebKit::SecurityOriginData> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("EstablishConnection"); }
    static const bool isSync = false;

    EstablishConnection(const String& databaseName, const WebKit::SecurityOriginData& openingOrigin, const WebKit::SecurityOriginData& mainFrameOrigin)
        : m_arguments(databaseName, openingOrigin, mainFrameOrigin)
    {
    }

    const std::tuple<const String&, const WebKit::SecurityOriginData&, const WebKit::SecurityOriginData&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<const String&, const WebKit::SecurityOriginData&, const WebKit::SecurityOriginData&> m_arguments;
};

class GetOrEstablishIDBDatabaseMetadata {
public:
    typedef std::tuple<uint64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("GetOrEstablishIDBDatabaseMetadata"); }
    static const bool isSync = false;

    explicit GetOrEstablishIDBDatabaseMetadata(uint64_t requestID)
        : m_arguments(requestID)
    {
    }

    const std::tuple<uint64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t> m_arguments;
};

class DeleteDatabase {
public:
    typedef std::tuple<uint64_t, String> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("DeleteDatabase"); }
    static const bool isSync = false;

    DeleteDatabase(uint64_t requestID, const String& databaseName)
        : m_arguments(requestID, databaseName)
    {
    }

    const std::tuple<uint64_t, const String&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, const String&> m_arguments;
};

class OpenTransaction {
public:
    typedef std::tuple<uint64_t, int64_t, Vector<int64_t>, uint64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("OpenTransaction"); }
    static const bool isSync = false;

    OpenTransaction(uint64_t requestID, int64_t transactionID, const Vector<int64_t>& objectStoreIDs, uint64_t transactionMode)
        : m_arguments(requestID, transactionID, objectStoreIDs, transactionMode)
    {
    }

    const std::tuple<uint64_t, int64_t, const Vector<int64_t>&, uint64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, const Vector<int64_t>&, uint64_t> m_arguments;
};

class BeginTransaction {
public:
    typedef std::tuple<uint64_t, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("BeginTransaction"); }
    static const bool isSync = false;

    BeginTransaction(uint64_t requestID, int64_t transactionID)
        : m_arguments(requestID, transactionID)
    {
    }

    const std::tuple<uint64_t, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t> m_arguments;
};

class CommitTransaction {
public:
    typedef std::tuple<uint64_t, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("CommitTransaction"); }
    static const bool isSync = false;

    CommitTransaction(uint64_t requestID, int64_t transactionID)
        : m_arguments(requestID, transactionID)
    {
    }

    const std::tuple<uint64_t, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t> m_arguments;
};

class ResetTransaction {
public:
    typedef std::tuple<uint64_t, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("ResetTransaction"); }
    static const bool isSync = false;

    ResetTransaction(uint64_t requestID, int64_t transactionID)
        : m_arguments(requestID, transactionID)
    {
    }

    const std::tuple<uint64_t, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t> m_arguments;
};

class RollbackTransaction {
public:
    typedef std::tuple<uint64_t, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("RollbackTransaction"); }
    static const bool isSync = false;

    RollbackTransaction(uint64_t requestID, int64_t transactionID)
        : m_arguments(requestID, transactionID)
    {
    }

    const std::tuple<uint64_t, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t> m_arguments;
};

class ResetTransactionSync {
public:
    typedef std::tuple<int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("ResetTransactionSync"); }
    static const bool isSync = true;

    struct DelayedReply : public ThreadSafeRefCounted<DelayedReply> {
        DelayedReply(PassRefPtr<IPC::Connection>, std::unique_ptr<IPC::MessageEncoder>);
        ~DelayedReply();

        bool send(bool success);

    private:
        RefPtr<IPC::Connection> m_connection;
        std::unique_ptr<IPC::MessageEncoder> m_encoder;
    };

    typedef IPC::Arguments<bool&> Reply;
    explicit ResetTransactionSync(int64_t transactionID)
        : m_arguments(transactionID)
    {
    }

    const std::tuple<int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<int64_t> m_arguments;
};

class RollbackTransactionSync {
public:
    typedef std::tuple<int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("RollbackTransactionSync"); }
    static const bool isSync = true;

    struct DelayedReply : public ThreadSafeRefCounted<DelayedReply> {
        DelayedReply(PassRefPtr<IPC::Connection>, std::unique_ptr<IPC::MessageEncoder>);
        ~DelayedReply();

        bool send(bool success);

    private:
        RefPtr<IPC::Connection> m_connection;
        std::unique_ptr<IPC::MessageEncoder> m_encoder;
    };

    typedef IPC::Arguments<bool&> Reply;
    explicit RollbackTransactionSync(int64_t transactionID)
        : m_arguments(transactionID)
    {
    }

    const std::tuple<int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<int64_t> m_arguments;
};

class ChangeDatabaseVersion {
public:
    typedef std::tuple<uint64_t, int64_t, uint64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("ChangeDatabaseVersion"); }
    static const bool isSync = false;

    ChangeDatabaseVersion(uint64_t requestID, int64_t transactionID, uint64_t newVersion)
        : m_arguments(requestID, transactionID, newVersion)
    {
    }

    const std::tuple<uint64_t, int64_t, uint64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, uint64_t> m_arguments;
};

class CreateObjectStore {
public:
    typedef std::tuple<uint64_t, int64_t, WebCore::IDBObjectStoreMetadata> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("CreateObjectStore"); }
    static const bool isSync = false;

    CreateObjectStore(uint64_t requestID, int64_t transactionID, const WebCore::IDBObjectStoreMetadata& objectStoreMetadata)
        : m_arguments(requestID, transactionID, objectStoreMetadata)
    {
    }

    const std::tuple<uint64_t, int64_t, const WebCore::IDBObjectStoreMetadata&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, const WebCore::IDBObjectStoreMetadata&> m_arguments;
};

class DeleteObjectStore {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("DeleteObjectStore"); }
    static const bool isSync = false;

    DeleteObjectStore(uint64_t requestID, int64_t transactionID, int64_t objectStoreID)
        : m_arguments(requestID, transactionID, objectStoreID)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t> m_arguments;
};

class ClearObjectStore {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("ClearObjectStore"); }
    static const bool isSync = false;

    ClearObjectStore(uint64_t requestID, int64_t transactionID, int64_t objectStoreID)
        : m_arguments(requestID, transactionID, objectStoreID)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t> m_arguments;
};

class CreateIndex {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t, WebCore::IDBIndexMetadata> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("CreateIndex"); }
    static const bool isSync = false;

    CreateIndex(uint64_t requestID, int64_t transactionID, int64_t objectStoreID, const WebCore::IDBIndexMetadata& indexMetadata)
        : m_arguments(requestID, transactionID, objectStoreID, indexMetadata)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t, const WebCore::IDBIndexMetadata&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t, const WebCore::IDBIndexMetadata&> m_arguments;
};

class DeleteIndex {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("DeleteIndex"); }
    static const bool isSync = false;

    DeleteIndex(uint64_t requestID, int64_t transactionID, int64_t objectStoreID, int64_t indexID)
        : m_arguments(requestID, transactionID, objectStoreID, indexID)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t, int64_t> m_arguments;
};

class PutRecord {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t, WebCore::IDBKeyData, IPC::DataReference, int64_t, Vector<int64_t>, Vector<Vector<WebCore::IDBKeyData>>> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("PutRecord"); }
    static const bool isSync = false;

    PutRecord(uint64_t requestID, int64_t transactionID, int64_t objectStoreID, const WebCore::IDBKeyData& key, const IPC::DataReference& value, int64_t putMode, const Vector<int64_t>& indexIDs, const Vector<Vector<WebCore::IDBKeyData>>& indexKeys)
        : m_arguments(requestID, transactionID, objectStoreID, key, value, putMode, indexIDs, indexKeys)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t, const WebCore::IDBKeyData&, const IPC::DataReference&, int64_t, const Vector<int64_t>&, const Vector<Vector<WebCore::IDBKeyData>>&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t, const WebCore::IDBKeyData&, const IPC::DataReference&, int64_t, const Vector<int64_t>&, const Vector<Vector<WebCore::IDBKeyData>>&> m_arguments;
};

class GetRecord {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t, int64_t, WebCore::IDBKeyRangeData, int64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("GetRecord"); }
    static const bool isSync = false;

    GetRecord(uint64_t requestID, int64_t transactionID, int64_t objectStoreID, int64_t indexID, const WebCore::IDBKeyRangeData& keyRange, int64_t cursorType)
        : m_arguments(requestID, transactionID, objectStoreID, indexID, keyRange, cursorType)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&, int64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&, int64_t> m_arguments;
};

class OpenCursor {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t, int64_t, int64_t, int64_t, int64_t, WebCore::IDBKeyRangeData> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("OpenCursor"); }
    static const bool isSync = false;

    OpenCursor(uint64_t requestID, int64_t transactionID, int64_t objectStoreID, int64_t indexID, int64_t cursorDirection, int64_t cursorType, int64_t taskType, const WebCore::IDBKeyRangeData& keyRange)
        : m_arguments(requestID, transactionID, objectStoreID, indexID, cursorDirection, cursorType, taskType, keyRange)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t, int64_t, int64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t, int64_t, int64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&> m_arguments;
};

class CursorAdvance {
public:
    typedef std::tuple<uint64_t, int64_t, uint64_t> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("CursorAdvance"); }
    static const bool isSync = false;

    CursorAdvance(uint64_t requestID, int64_t cursorID, uint64_t count)
        : m_arguments(requestID, cursorID, count)
    {
    }

    const std::tuple<uint64_t, int64_t, uint64_t>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, uint64_t> m_arguments;
};

class CursorIterate {
public:
    typedef std::tuple<uint64_t, int64_t, WebCore::IDBKeyData> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("CursorIterate"); }
    static const bool isSync = false;

    CursorIterate(uint64_t requestID, int64_t cursorID, const WebCore::IDBKeyData& key)
        : m_arguments(requestID, cursorID, key)
    {
    }

    const std::tuple<uint64_t, int64_t, const WebCore::IDBKeyData&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, const WebCore::IDBKeyData&> m_arguments;
};

class Count {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t, int64_t, WebCore::IDBKeyRangeData> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("Count"); }
    static const bool isSync = false;

    Count(uint64_t requestID, int64_t transactionID, int64_t objectStoreID, int64_t indexID, const WebCore::IDBKeyRangeData& keyRange)
        : m_arguments(requestID, transactionID, objectStoreID, indexID, keyRange)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&> m_arguments;
};

class DeleteRange {
public:
    typedef std::tuple<uint64_t, int64_t, int64_t, WebCore::IDBKeyRangeData> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("DeleteRange"); }
    static const bool isSync = false;

    DeleteRange(uint64_t requestID, int64_t transactionID, int64_t objectStoreID, const WebCore::IDBKeyRangeData& keyRange)
        : m_arguments(requestID, transactionID, objectStoreID, keyRange)
    {
    }

    const std::tuple<uint64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<uint64_t, int64_t, int64_t, const WebCore::IDBKeyRangeData&> m_arguments;
};

class Close {
public:
    typedef std::tuple<> DecodeType;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("Close"); }
    static const bool isSync = false;

    const std::tuple<>& arguments() const
    {
        return m_arguments;
    }

private:
    std::tuple<> m_arguments;
};

} // namespace DatabaseProcessIDBConnection
} // namespace Messages

#endif // ENABLE(INDEXED_DATABASE) && ENABLE(DATABASE_PROCESS)

#endif // DatabaseProcessIDBConnectionMessages_h
