/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#if ENABLE(INDEXED_DATABASE) && ENABLE(DATABASE_PROCESS)

#include "WebIDBServerConnection.h"

#include "ArgumentCoders.h"
#include "DataReference.h"
#include "HandleMessage.h"
#include "MessageDecoder.h"
#include "WebCoreArgumentCoders.h"
#include "WebIDBServerConnectionMessages.h"
#include <WebCore/IDBDatabaseMetadata.h>
#include <WebCore/IDBGetResult.h>
#include <WebCore/IDBKeyData.h>
#include <wtf/text/WTFString.h>

namespace WebKit {

void WebIDBServerConnection::didReceiveWebIDBServerConnectionMessage(IPC::Connection& connection, IPC::MessageDecoder& decoder)
{
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidGetOrEstablishIDBDatabaseMetadata::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidGetOrEstablishIDBDatabaseMetadata>(decoder, this, &WebIDBServerConnection::didGetOrEstablishIDBDatabaseMetadata);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidDeleteDatabase::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidDeleteDatabase>(decoder, this, &WebIDBServerConnection::didDeleteDatabase);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidOpenTransaction::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidOpenTransaction>(decoder, this, &WebIDBServerConnection::didOpenTransaction);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidBeginTransaction::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidBeginTransaction>(decoder, this, &WebIDBServerConnection::didBeginTransaction);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidCommitTransaction::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidCommitTransaction>(decoder, this, &WebIDBServerConnection::didCommitTransaction);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidResetTransaction::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidResetTransaction>(decoder, this, &WebIDBServerConnection::didResetTransaction);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidRollbackTransaction::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidRollbackTransaction>(decoder, this, &WebIDBServerConnection::didRollbackTransaction);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidChangeDatabaseVersion::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidChangeDatabaseVersion>(decoder, this, &WebIDBServerConnection::didChangeDatabaseVersion);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidCreateObjectStore::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidCreateObjectStore>(decoder, this, &WebIDBServerConnection::didCreateObjectStore);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidDeleteObjectStore::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidDeleteObjectStore>(decoder, this, &WebIDBServerConnection::didDeleteObjectStore);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidClearObjectStore::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidClearObjectStore>(decoder, this, &WebIDBServerConnection::didClearObjectStore);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidCreateIndex::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidCreateIndex>(decoder, this, &WebIDBServerConnection::didCreateIndex);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidDeleteIndex::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidDeleteIndex>(decoder, this, &WebIDBServerConnection::didDeleteIndex);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidPutRecord::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidPutRecord>(decoder, this, &WebIDBServerConnection::didPutRecord);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidGetRecord::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidGetRecord>(decoder, this, &WebIDBServerConnection::didGetRecord);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidOpenCursor::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidOpenCursor>(decoder, this, &WebIDBServerConnection::didOpenCursor);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidAdvanceCursor::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidAdvanceCursor>(decoder, this, &WebIDBServerConnection::didAdvanceCursor);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidIterateCursor::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidIterateCursor>(decoder, this, &WebIDBServerConnection::didIterateCursor);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidCount::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidCount>(decoder, this, &WebIDBServerConnection::didCount);
        return;
    }
    if (decoder.messageName() == Messages::WebIDBServerConnection::DidDeleteRange::name()) {
        IPC::handleMessage<Messages::WebIDBServerConnection::DidDeleteRange>(decoder, this, &WebIDBServerConnection::didDeleteRange);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

#endif // ENABLE(INDEXED_DATABASE) && ENABLE(DATABASE_PROCESS)
